// ---------------------------------------------------------------------------
// - cucd.hxx                                                                -
// - unicode database (ucd) library private definitions                      -
// ---------------------------------------------------------------------------
// - This program is free software;  you can redistribute it  and/or  modify -
// - it provided that this copyright notice is kept intact.                  -
// -                                                                         -
// - This program  is  distributed in  the hope  that it will be useful, but -
// - without  any  warranty;  without  even   the   implied    warranty   of -
// - merchantability or fitness for a particular purpose.  In no event shall -
// - the copyright holder be liable for any  direct, indirect, incidental or -
// - special damages arising in any way out of the use of this software.     -
// ---------------------------------------------------------------------------
// - copyright (c) 1999-2006 amaury darsch                                   -
// ---------------------------------------------------------------------------

#include "cucd.hpp"

namespace afnix {

  // plane 0 access function
  const long   ucd_get_psize_0000 (void);
  const ucd_s* ucd_get_plane_0000 (void);

  // plane 1 access function
  const long   ucd_get_psize_0001 (void);
  const ucd_s* ucd_get_plane_0001 (void);

  // plane 2 access function
  const long   ucd_get_psize_0002 (void);
  const ucd_s* ucd_get_plane_0002 (void);

  // plane 3 access function
  const long   ucd_get_psize_0003 (void);
  const ucd_s* ucd_get_plane_0003 (void);

  // plane 4 access function
  const long   ucd_get_psize_0004 (void);
  const ucd_s* ucd_get_plane_0004 (void);

  // plane 5 access function
  const long   ucd_get_psize_0005 (void);
  const ucd_s* ucd_get_plane_0005 (void);

  // plane 6 access function
  const long   ucd_get_psize_0006 (void);
  const ucd_s* ucd_get_plane_0006 (void);

  // plane 7 access function
  const long   ucd_get_psize_0007 (void);
  const ucd_s* ucd_get_plane_0007 (void);

  // plane 8 access function
  const long   ucd_get_psize_0008 (void);
  const ucd_s* ucd_get_plane_0008 (void);

  // plane 9 access function
  const long   ucd_get_psize_0009 (void);
  const ucd_s* ucd_get_plane_0009 (void);

  // plane A access function
  const long   ucd_get_psize_000A (void);
  const ucd_s* ucd_get_plane_000A (void);

  // plane B access function
  const long   ucd_get_psize_000B (void);
  const ucd_s* ucd_get_plane_000B (void);

  // plane C access function
  const long   ucd_get_psize_000C (void);
  const ucd_s* ucd_get_plane_000C (void);

  // plane D access function
  const long   ucd_get_psize_000D (void);
  const ucd_s* ucd_get_plane_000D (void);

  // plane E access function
  const long   ucd_get_psize_000E (void);
  const ucd_s* ucd_get_plane_000E (void);
};
